@@ -138,13 +138,13 @@ sub send {
 	my $mask = $self->mask ||= 0;
 	my (@ENCODED) = map { ord($_) } split //, $message;
 	my $length = scalar @ENCODED + 128;
-	if ($length > 256 || $mask) {
+	if ($length > 254 || $mask) {
 		@MASK = map { int(rand(256)) } 0 .. 3;
 		my $i;
 		$ENCODED[$i++] = $_ ^ $MASK[$i % 4] for (@ENCODED);
 		unshift @ENCODED, @MASK;
 		if ($length > 256) {
-			my $times = int($length / 256);
+			my $times = int(($length + 2) / 254) - 1;
 			my $excess = $length - (($times * 256) + 128);
 			unshift @ENCODED, (254, $times, $excess);
 		} else {
@@ -158,28 +158,37 @@ sub send {
 }
 
 sub recieve {
-	my ($self) = @_;
-	my $pg = $self->psgix;
-	my $content = "";
-	my $len = sysread($pg, $content, 8192);
-	return unless $len;
-	$len = sysread($pg, $content, 8192, length($content)) while $len >= 8192;
-	my @bits = split //, unpack("B*", substr($content, 0, 1));
+	my ($self, @ENCODED) = @_;
+	my $length;
+	if (! scalar @ENCODED ) {
+		return shift @{ $self->next_frame } if scalar @{ $self->next_frame ||= [] }; 
+		my $pg = $self->psgix;
+		my $content = "";
+		$length = sysread($pg, $content, 8192);
+		return unless $length;
+		$length = sysread($pg, $content, 8192, length($content)) while $length >= 8192;
+		@ENCODED = map { unpack "C", $_ } split //, $content;
+	}
+	my @bits = split //, sprintf("%b\n", $ENCODED[0]);
         $self->fin = $bits[0];
         $self->rsv = [@bits[1 .. 3]];
-	my @ENCODED = map { unpack "C", $_ } split //, $content;
 	$self->op = shift @ENCODED;
-	my $length;
-	if (scalar @ENCODED > 127) {
+	if ($ENCODED[0] == 254) {
 		my @length = splice @ENCODED, 0, 3;
 		$length = ((($length[0] + 2) * $length[1]) + $length[2]);
 	} else {
 		$length = shift @ENCODED;
 		$length -= 128;
 	}
+	return pack "C*", join("", @ENCODED) if (scalar @ENCODED == $length);
 	my @MASK = splice @ENCODED, 0, 4;
+	if (scalar @ENCODED > $length) {
+		my $next = $self->recieve(splice @ENCODED, $length, scalar @ENCODED);
+		return $next if ($next eq 'invalid_length');
+		unshift @{ $self->next_frame }, $next;
+	}
 	return 'invalid_length' if (scalar @ENCODED != $length);
-	return join "", map { chr($ENCODED[$_] ^ $MASK[$_ ^ 4]) } 0 .. scalar @ENCOEDED;
+	return join "", map { pack "C", ($ENCODED[$_] ^ $MASK[$_ % 4]) } 0 .. $#ENCODED;
 }
 
 __DONE__;
@@ -192,7 +201,7 @@ Terse::WebSocket - Lightweight WebSockets
 
 =head1 VERSION
 
-Version 0.115
+Version 0.116
 
 =cut
 
